@@ -178,7 +178,10 @@ func isRequiredConfigPresent(config []byte) error {
 func createTargetConfigController(c TargetConfigController, recorder events.Recorder, operatorSpec *operatorv1.StaticPodOperatorSpec) (bool, error) {
 	errors := []error{}
 
-	directResourceResults := resourceapply.ApplyDirectly(c.kubeClient, c.eventRecorder, v411_00_assets.Asset,
+	directResourceResults := resourceapply.ApplyDirectly(
+		resourceapply.NewKubeClientHolder(c.kubeClient),
+		c.eventRecorder,
+		v411_00_assets.Asset,
 		"v4.1.0/kube-controller-manager/ns.yaml",
 		"v4.1.0/kube-controller-manager/kubeconfig-cert-syncer.yaml",
 		"v4.1.0/kube-controller-manager/kubeconfig-cm.yaml",
@@ -189,8 +192,9 @@ func createTargetConfigController(c TargetConfigController, recorder events.Reco
 		"v4.1.0/kube-controller-manager/leader-election-kube-controller-manager-rolebinding-kube-system.yaml",
 		"v4.1.0/kube-controller-manager/svc.yaml",
 		"v4.1.0/kube-controller-manager/sa.yaml",
-		"v4.1.0/kube-controller-manager/gce/cloud-provider-role.yaml",
-		"v4.1.0/kube-controller-manager/gce/cloud-provider-binding.yaml",
+		"v4.1.0/kube-controller-manager/localhost-recovery-client-crb.yaml",
+		"v4.1.0/kube-controller-manager/localhost-recovery-sa.yaml",
+		"v4.1.0/kube-controller-manager/localhost-recovery-token.yaml",
 	)
 	for _, currResult := range directResourceResults {
 		if currResult.Error != nil {
@@ -206,15 +210,15 @@ func createTargetConfigController(c TargetConfigController, recorder events.Reco
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "configmap/cluster-policy-controller-config", err))
 	}
-	_, _, err = manageCSRIntermediateCABundle(c.secretLister, c.kubeClient.CoreV1(), recorder)
+	_, _, err = ManageCSRIntermediateCABundle(c.secretLister, c.kubeClient.CoreV1(), recorder)
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "configmap/csr-intermediate-ca", err))
 	}
-	_, _, err = manageCSRCABundle(c.configMapLister, c.kubeClient.CoreV1(), recorder)
+	_, _, err = ManageCSRCABundle(c.configMapLister, c.kubeClient.CoreV1(), recorder)
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "configmap/csr-controller-ca", err))
 	}
-	_, requeueDelay, _, err := manageCSRSigner(c.secretLister, c.kubeClient.CoreV1(), recorder)
+	_, requeueDelay, _, err := ManageCSRSigner(c.secretLister, c.kubeClient.CoreV1(), recorder)
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "secrets/csr-signer", err))
 	}
@@ -225,6 +229,10 @@ func createTargetConfigController(c TargetConfigController, recorder events.Reco
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "configmap/serviceaccount-ca", err))
 	}
+	err = ensureLocalhostRecoverySAToken(c.kubeClient.CoreV1(), recorder)
+	if err != nil {
+		errors = append(errors, fmt.Errorf("%q: %v", "serviceaccount/localhost-recovery-client", err))
+	}
 	_, _, err = managePod(c.kubeClient.CoreV1(), c.kubeClient.CoreV1(), recorder, operatorSpec, c.targetImagePullSpec, c.operatorImagePullSpec, c.clusterPolicyControllerPullSpec)
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "configmap/kube-controller-manager-pod", err))
@@ -293,6 +301,55 @@ func manageClusterPolicyControllerConfig(client corev1client.ConfigMapsGetter, r
 	return resourceapply.ApplyConfigMap(client, recorder, requiredConfigMap)
 }
 
+func ensureLocalhostRecoverySAToken(client corev1client.CoreV1Interface, recorder events.Recorder) error {
+	requiredSA := resourceread.ReadServiceAccountV1OrDie(v411_00_assets.MustAsset("v4.1.0/kube-controller-manager/localhost-recovery-sa.yaml"))
+	requiredToken := resourceread.ReadSecretV1OrDie(v411_00_assets.MustAsset("v4.1.0/kube-controller-manager/localhost-recovery-token.yaml"))
+
+	saClient := client.ServiceAccounts(operatorclient.TargetNamespace)
+	serviceAccount, err := saClient.Get(requiredSA.Name, metav1.GetOptions{})
+	if err != nil {
+		return err
+	}
+
+	// The default token secrets get random names so we have created a custom secret
+	// to be populated with SA token so we have a stable name.
+	secretsClient := client.Secrets(operatorclient.TargetNamespace)
+	token, err := secretsClient.Get(requiredToken.Name, metav1.GetOptions{})
+	if err != nil {
+		return err
+	}
+
+	// Token creation / injection for a SA is asynchronous.
+	// We will report and error if it's missing, go degraded and get re-queued when the SA token is updated.
+
+	uid := token.Annotations[corev1.ServiceAccountUIDKey]
+	if len(uid) == 0 {
+		return fmt.Errorf("secret %s/%s hasn't been populated with SA token yet: missing SA UID", token.Namespace, token.Name)
+	}
+
+	if uid != string(serviceAccount.UID) {
+		return fmt.Errorf("secret %s/%s hasn't been populated with current SA token yet: SA UID mismatch", token.Namespace, token.Name)
+	}
+
+	if len(token.Data) == 0 {
+		return fmt.Errorf("secret %s/%s hasn't been populated with any data yet", token.Namespace, token.Name)
+	}
+
+	// Explicitly check that the fields we use are there, so we find out easily if some are removed or renamed.
+
+	_, ok := token.Data["token"]
+	if !ok {
+		return fmt.Errorf("secret %s/%s hasn't been populated with current SA token yet", token.Namespace, token.Name)
+	}
+
+	_, ok = token.Data["ca.crt"]
+	if !ok {
+		return fmt.Errorf("secret %s/%s hasn't been populated with current SA token root CA yet", token.Namespace, token.Name)
+	}
+
+	return err
+}
+
 func managePod(configMapsGetter corev1client.ConfigMapsGetter, secretsGetter corev1client.SecretsGetter, recorder events.Recorder, operatorSpec *operatorv1.StaticPodOperatorSpec, imagePullSpec, operatorImagePullSpec, clusterPolicyControllerPullSpec string) (*corev1.ConfigMap, bool, error) {
 	required := resourceread.ReadPodV1OrDie(v411_00_assets.MustAsset("v4.1.0/kube-controller-manager/pod.yaml"))
 	// TODO: If the image pull spec is not specified, the "${IMAGE}" will be used as value and the pod will fail to start.
@@ -387,7 +444,7 @@ func manageServiceAccountCABundle(lister corev1listers.ConfigMapLister, client c
 	return resourceapply.ApplyConfigMap(client, recorder, requiredConfigMap)
 }
 
-func manageCSRCABundle(lister corev1listers.ConfigMapLister, client corev1client.ConfigMapsGetter, recorder events.Recorder) (*corev1.ConfigMap, bool, error) {
+func ManageCSRCABundle(lister corev1listers.ConfigMapLister, client corev1client.ConfigMapsGetter, recorder events.Recorder) (*corev1.ConfigMap, bool, error) {
 	requiredConfigMap, err := resourcesynccontroller.CombineCABundleConfigMaps(
 		resourcesynccontroller.ResourceLocation{Namespace: operatorclient.OperatorNamespace, Name: "csr-controller-ca"},
 		lister,
@@ -402,7 +459,7 @@ func manageCSRCABundle(lister corev1listers.ConfigMapLister, client corev1client
 	return resourceapply.ApplyConfigMap(client, recorder, requiredConfigMap)
 }
 
-func manageCSRSigner(lister corev1listers.SecretLister, client corev1client.SecretsGetter, recorder events.Recorder) (*corev1.Secret, time.Duration, bool, error) {
+func ManageCSRSigner(lister corev1listers.SecretLister, client corev1client.SecretsGetter, recorder events.Recorder) (*corev1.Secret, time.Duration, bool, error) {
 	// get the certkey pair we will sign with. We're going to add the cert to a ca bundle so we can recognize the chain it signs back to the signer
 	csrSigner, err := lister.Secrets(operatorclient.OperatorNamespace).Get("csr-signer")
 	if apierrors.IsNotFound(err) {
@@ -413,45 +470,76 @@ func manageCSRSigner(lister corev1listers.SecretLister, client corev1client.Secr
 	}
 
 	// the CSR signing controller only accepts a single cert.  make sure we only ever have one (not multiple to construct a larger chain)
-	signingCert := csrSigner.Data["tls.crt"]
-	if len(signingCert) == 0 {
-		return nil, 0, false, nil
-	}
-	signingKey := csrSigner.Data["tls.key"]
-	if len(signingCert) == 0 {
-		return nil, 0, false, nil
-	}
-	signingCertKeyPair, err := crypto.GetCAFromBytes(signingCert, signingKey)
-	if err != nil {
-		return nil, 0, false, err
-	}
-	certBytes, err := crypto.EncodeCertificates(signingCertKeyPair.Config.Certs[0])
-	if err != nil {
+	certBytes, signingKey, useAfter, _, err := extractSigner(csrSigner)
+	if certBytes == nil || signingKey == nil || err != nil {
 		return nil, 0, false, err
 	}
 
 	// make sure we wait five minutes to propagate the change to other components, like kas for trust
-	useAfter := signingCertKeyPair.Config.Certs[0].NotBefore.Add(5 * time.Minute)
+	useAfter = useAfter.Add(5 * time.Minute)
 	now := time.Now()
-	if useAfter.Before(now) {
-		// if we have something and it's not expired (yeah that check is missing here), delay
-		if _, err := client.Secrets(operatorclient.TargetNamespace).Get("csr-signer", metav1.GetOptions{}); err == nil {
-			return nil, useAfter.Sub(now) + 10*time.Second, false, nil
-		}
+
+	oldSigner, err := client.Secrets(operatorclient.TargetNamespace).Get("csr-signer", metav1.GetOptions{})
+	_, _, _, oldUseBefore, _ := extractSigner(oldSigner)
+	switch {
+	case apierrors.IsNotFound(err):
+		// apply the secret
+
+	case oldUseBefore.Before(now):
+		// apply the secret
+
+	case now.After(useAfter):
+		// apply the secret
+
+	default:
+		// wait a little while longer until after the useAfter
+		return nil, useAfter.Sub(now) + 10*time.Second, false, nil
 	}
 
 	csrSigner = &corev1.Secret{
 		ObjectMeta: metav1.ObjectMeta{Namespace: operatorclient.TargetNamespace, Name: "csr-signer"},
 		Data: map[string][]byte{
 			"tls.crt": certBytes,
-			"tls.key": []byte(signingKey),
+			"tls.key": signingKey,
 		},
+		Type: corev1.SecretTypeOpaque,
 	}
 	secret, modified, err := resourceapply.ApplySecret(client, recorder, csrSigner)
 	return secret, 0, modified, err
 }
 
-func manageCSRIntermediateCABundle(lister corev1listers.SecretLister, client corev1client.ConfigMapsGetter, recorder events.Recorder) (*corev1.ConfigMap, bool, error) {
+func extractSigner(csrSigner *corev1.Secret) ([]byte, []byte, time.Time, time.Time, error) {
+	useAfter := time.Unix(0, 0)
+	useBefore := time.Unix(0, 0)
+
+	if csrSigner == nil {
+		return nil, nil, useAfter, useBefore, nil
+	}
+
+	signingCert := csrSigner.Data["tls.crt"]
+	if len(signingCert) == 0 {
+		return nil, nil, useAfter, useBefore, nil
+	}
+	signingKey := csrSigner.Data["tls.key"]
+	if len(signingKey) == 0 {
+		return nil, nil, useAfter, useBefore, nil
+	}
+	signingCertKeyPair, err := crypto.GetCAFromBytes(signingCert, signingKey)
+	if err != nil {
+		return nil, nil, useAfter, useBefore, err
+	}
+	certBytes, err := crypto.EncodeCertificates(signingCertKeyPair.Config.Certs[0])
+	if err != nil {
+		return nil, nil, useAfter, useBefore, err
+	}
+
+	useAfter = signingCertKeyPair.Config.Certs[0].NotBefore
+	useBefore = signingCertKeyPair.Config.Certs[0].NotAfter
+
+	return certBytes, signingKey, useAfter, useBefore, nil
+}
+
+func ManageCSRIntermediateCABundle(lister corev1listers.SecretLister, client corev1client.ConfigMapsGetter, recorder events.Recorder) (*corev1.ConfigMap, bool, error) {
 	// get the certkey pair we will sign with. We're going to add the cert to a ca bundle so we can recognize the chain it signs back to the signer
 	csrSigner, err := lister.Secrets(operatorclient.OperatorNamespace).Get("csr-signer")
 	if apierrors.IsNotFound(err) {
